<div class="row img-rounded">
    <hr><small>
<div class="col-md-4">
</div>
<div class="col-md-4">
</div>
<div class="col-md-4">
    <a class="pull-right" href="https://github.com/dpgaspar/Flask-AppBuilder">Powered by F.A.B. (dpgaspar).</a>
</div>
</div>
