.navbar .brand { 
	max-height: 40px; 
	overflow: visible; 
	padding-top: 0; 
	padding-bottom: 0; 
}

.collapse.in {
    display:block;
	overflow: visible;
}

.panel-group .panel {
    overflow: visible;
}

/* Wrapper for page content to push down footer */
.wrap {
	min-height: 100%;
	height: auto !important;
	height: 100%;
}


.fixed-footer{
	background-color: #f5f5f5;
	bottom:0;
	left:0;
	right:0;	
	width:100%;
	margin:auto;
}
