{% import 'appbuilder/general/lib.html' as lib %}


	{% set can_add = "can_add" | is_item_visible(generalview_name) %}
	{% set can_show = "can_show" | is_item_visible(generalview_name) %}
    {% set can_edit = "can_edit" | is_item_visible(generalview_name) %}
	{% set can_delete = "can_delete" | is_item_visible(generalview_name) %}

	{{ lib.render_list_header(can_add, page, page_size, count, filters, generalview_name) }}

	{% if value_columns %}
	
	<div class="container">
      <div class="row">
		
		
  {% for item in value_columns %}
  	<div class="col-md-4">
  	<div class="thumbnail">
		<div class="list-group">
		{% if can_show or can_edit or can_delete %}
		    {% set pk = pks[loop.index-1] %}
            {{ lib.btn_crud(can_show, can_edit, can_delete, pk, generalview_name, filters) }}
		{% endif %}
		</div>
		  
        {% for value in include_columns %}
            {% if loop.index == 1 %}
                <h4 class="list-group-item-heading">{{ item[value]|safe }}</h4>
            {% else %}
                <p class="list-group-item-text">{{ item[value]|safe }}</p>
            {% endif %}
		{% endfor %}
	  </div>
	  </div>
  {% endfor %}

	  </div>
		</div>
		
	{% else %}
	<b> {{_("No records found")}}</b>
	{% endif %}
